# 招投标数据采集系统

## 项目说明

本系统用于自动采集和处理招投标信息，主要功能包括：

- 数据采集：从多个招投标网站自动采集数据
- 数据处理：使用 AI 处理和分析采集的数据
- 数据分发：通过邮件发送处理后的数据
- 数据清理：定期清理历史数据

## 系统架构

### 核心模块

- 数据采集模块 (DataCollector)
- 数据处理模块 (DataProcess)
- 数据发送模块 (DataSend)
- 数据清理模块 (DataClean)

### 关键组件

- 适配器 (Adapters): 负责对接不同的数据源
- 存储层 (Stores): 负责数据持久化
- 工具类 (Utils): 提供通用功能支持

## 配置说明

### 基础配置

```yaml
# 数据采集配置
save:
  collect_data_key: '红外光谱仪,拉曼光谱仪' # 采集数据关键词过滤
  collect_batch_size: 100 # 采集数据批量保存大小
  process_batch_size: 1 # AI处理数据批量大小
  attach_file_path: './temp_files/attaches/' # 附件保存路径
  report_file_path: './temp_files/report/' # 报表保存路径

# 任务调度配置
job:
  event_id: 1 # 任务ID,改变此值会触发任务重新加载
  sleep_interval: 10 # 任务检查间隔(秒)
  collect: '06:00,22:00' # 每天数据采集时间点
  process: '07:00,10:00' # 每天数据处理时间点
  send_email: '08:20,14:00' # 每天邮件发送时间点
  clean_data: '00:05' # 每天数据清理时间点

  # 月度报告配置
  send_current_month_report_day: 30 # 每月发送当月报告的日期
  send_current_month_report_time: '08:20' # 发送当月报告的时间点
  send_prev_month_report_day: 1 # 每月发送上月报告的日期
  send_prev_month_report_time: '08:20' # 发送上月报告的时间点

  run_now: false # 是否立即执行一次任务

# 数据清理配置
clean:
  day: 30 # 默认清理天数(所有类型默认使用此值)
  attach: 30 # 附件清理天数
  log: 30 # 日志清理天数
  collect_data: 30 # 采集数据清理天数
  process_data: 30 # 招标数据清理天数
  process_result_data: 60 # 中标数据清理天数(最小45天)
  report: 90 # 报表清理天数(最小60天)

# AI配置
ai:
  key: 'your-api-key' # AI API密钥
  url: 'https://api-url' # AI API地址
  model: 'model-name' # 使用的模型名称
  # 系统提示词
  system_prompt: '从给定信息中提取出关键信息，并以给定的类型返回json字符串，如果部分信息为空，则该字段返回为空' 
  # 模板1 处理招标信息
  prompt_template_1: '在以上内容中提取招标信息：
    ```typescript
    export interface Tender { //招标信息
    no: string; // 招标项目编号
    title: string; // 招标公告标题
    province: string; // 招标单位省份
    city: string; // 招标单位城市
    date: string; // 项目开标的时间
    address: string; // 项目开标的地点
    release_date: string; // 招标信息的发布时间
    summary: string; // 100字左右的招标条件，联系方式等内容摘要
    devices: string; // 只涉及到光谱仪相关的设备，其他无关设备不需要，多个设备以逗号分割 ，例如 红外光谱仪,拉曼光谱仪等
    }
    ```' 
  # 模板2 处理中标信息
  prompt_template_2: '在以上内容中提取中标信息：
    ```typescript
    export interface Instrument { // 中标仪器信息
    company: string; // 中标单位名称，参与竞标并中标的公司名称
    name: string; // 仪器名称，例如：红外光谱仪
    manufacturer: string; // 仪器厂商，例如：赛默飞、Bruker
    model: string; // 仪器的型号/规格，例如：NIR25S
    quantity: number; // 中标仪器的数量，台数，例如：2
    unit_price: number; // 仪器的单价，单位转换为元，例如：178000.00
    }
    export interface BiddingAcceptance { //中标信息
    no: string; // 项目编号
    title: string; // 中标公告标题
    date: string; // 中标公告时间
    province: string; // 招标单位省份
    city: string; // 招标单位城市
    summary: string; // 公告摘要信息，100字左右
    instruments: Instrument[]; // 中标设备的信息
    }
    ```'
# 邮件配置
email:
  smtp_server: 'smtp.example.com' # SMTP服务器地址
  smtp_port: 465 # SMTP端口
  smtp_user: 'user@example.com' # SMTP用户名
  smtp_password: 'password' # SMTP密码
  from_email: 'from@example.com' # 发件人地址
  error_email: 'error@example.com' # 错误通知邮箱
  default_email: 'default@example.com' # 默认收件人地址，所有业务邮件都发送到该地址

# 数据库配置  可以在docker-compose中的环境变量中配置
mysql:
  host: 'localhost' # 数据库主机
  port: 3306 # 数据库端口
  db: 'database_name' # 数据库名
  user: 'root' # 数据库用户名
  password: 'password' # 数据库密码
  charset: 'utf8mb4' # 字符集
```

### 数据源配置

```yaml
adapter:
  max_retries: 3 # 最大重试次数
  # 中国政府采购网配置
  ccgp:
    search_day: '近3日' # 搜索时间范围
    model_name: 'ccgp_data_collection_adapter' # 适配器模块名
    class_name: 'CCGPDataCollectionAdapter' # 适配器类名
    batch_save: false # 是否批量保存数据

  # 中国采购与招标网配置
  chinabidding:
    search_day: '近一周' # 搜索时间范围
    model_name: 'chinabidding_data_collection_adapter' # 适配器模块名
    class_name: 'ChinabiddingDataCollectionAdapter' # 适配器类名
    batch_save: true # 是否批量保存数据

# Selenium配置
selenium:
  remote_driver_url: 'http://127.0.0.1:3534/wd/hub' # WebDriver地址
```

### 日志配置

```yaml
logger:
  file-path: './logs/' # 日志文件路径
  level: 'debug' # 日志级别
```

## 部署说明

### 环境要求

- Python 3.8+
- MySQL 5.7+
- Selenium WebDriver

### 安装步骤

1. 安装依赖: `pip install -r requirements.txt`
2. 配置数据库: 执行 `init.sql`
3. 修改配置: 编辑 `config.yml`
4. 启动应用: `python app/main.py`

### 目录结构

```
app/
├── adapters/           # 数据源适配器
├── drivers/            # 浏览器驱动
├── jobs/              # 任务处理模块
├── models/            # 数据模型
├── stores/            # 数据存储
├── utils/             # 工具类
├── config.yml         # 配置文件
└── main.py           # 主程序
```

## 数据采集说明

### 采集流程

1. 系统按配置的时间点(`job.collect`)自动启动采集任务
2. 根据配置的数据源(`adapter`)和关键词(`save.collect_data_key`)进行数据采集
3. 采集到的数据经过关键词过滤后保存到数据库
4. 相关附件下载到指定目录(`save.attach_file_path`)



