from abc import ABC, abstractmethod
from typing import Optional, List, Any
from datetime import datetime

from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.common.exceptions import TimeoutException

import drivers
import utils
from models.collect_data import CollectData
from stores.data_store_interface import IDataStore


class IDataCollectionAdapter(ABC):
    """数据采集适配器基类"""

    def __init__(self, url: str, adapter_type: str, default_search_txt: str = ""):
        """初始化适配器

        Args:
            url: 目标网站URL
        """
        self._url = url
        self._adapter_type = adapter_type
        self._default_search_txt = default_search_txt
        self._search_txt = utils.get_config_value(
            self.search_day_key, default_search_txt
        )
        self._next_count = 0

        self._store: Optional[IDataStore] = None
        self._driver: Optional[WebDriver] = None
        self._adapter_type = ""
        self._keyword = None
        self._keywords: List[str] = []
        self._timeout = utils.get_config_int("selenium.page_load_timeout", 30)
        self._max_retries = utils.get_config_int("adapter.max_retries", 3)
        self._retry_keywords: dict = {}  # 记录重试关键词及次数

        try:
            # 初始化WebDriver
            self._driver = drivers.gen_driver(url)
            self.logger.info(f"初始化WebDriver成功: {url}")
        except Exception as e:
            self.logger.error(f"初始化WebDriver失败: {e}")
            raise

    @property
    def logger(self):
        return utils.get_logger()

    @property
    def driver(self) -> WebDriver:
        return self._driver

    @property
    def store(self) -> IDataStore:
        return self._store

    @property
    def url(self) -> str:
        return self._url

    @property
    def keyword(self) -> str:
        return self._keyword

    @property
    def adapter_type(self) -> str:
        return self._adapter_type

    @property
    def batch_save_key(self) -> str:
        return f"adapter.{self._adapter_type}.batch_save"

    @property
    def search_day_key(self) -> str:
        return f"adapter.{self._adapter_type}.search_day"

    def collect(self, keyword: str, store: Optional[IDataStore] = None) -> None:
        """执行数据采集

        Args:
            keyword: 搜索关键词,多个关键词以逗号分隔
            store: 数据存储器
        """
        try:
            self._store = store
            self._keyword = keyword
            self._keywords = utils.to_array(keyword)
            self._retry_keywords.clear()

            # 首次采集所有关键词
            for kw in self._keywords:
                try:
                    self.logger.debug(f"开始采集关键词: {kw}")
                    self._collect(kw)
                except TimeoutException as e:
                    self.logger.warning(f"采集关键词 {kw} 超时: {e}")
                    self._retry_keywords[kw] = 1
                    continue
                except Exception as e:
                    self.logger.error(f"采集关键词 {kw} 失败: {e}")
                    continue

            # 重试超时的关键词
            while self._retry_keywords:
                retry_kws = list(self._retry_keywords.keys())
                for kw in retry_kws:
                    retry_count = self._retry_keywords[kw]
                    if retry_count >= self._max_retries:
                        self.logger.error(f"关键词 {kw} 超过最大重试次数")
                        del self._retry_keywords[kw]
                        continue

                    try:
                        self.logger.info(f"重试采集关键词[{retry_count}]: {kw}")
                        self._collect(kw)
                        del self._retry_keywords[kw]
                    except TimeoutException as e:
                        self.logger.warning(f"重试采集关键词 {kw} 超时: {e}")
                        self._retry_keywords[kw] = retry_count + 1
                        continue
                    except Exception as e:
                        self.logger.error(f"重试采集关键词 {kw} 失败: {e}")
                        del self._retry_keywords[kw]
                        continue

            # 批量保存
            if utils.get_config_bool(self.batch_save_key):
                self.store.save_collect_data(True)

        except Exception as e:
            self.logger.error(f"采集失败: {e}")
            raise
        finally:
            self.cleanup()

    def _wait_for(
        self,
        condition: Any,
        timeout: Optional[int] = None,
        message: Optional[str] = None,
    ) -> Any:
        """等待条件满足

        Args:
            condition: 等待条件
            timeout: 超时时间(秒),默认使用全局超时时间
            message: 超时错误消息

        Returns:
            Any: 条件满足时的返回值

        Raises:
            TimeoutException: 等待超时
        """
        if not timeout:
            timeout = self._timeout

        wait = WebDriverWait(self.driver, timeout)
        return wait.until(condition, message)

    def _check_is_collect_by_url(self, url: str) -> bool:
        """检查URL是否已采集

        Args:
            url: 目标URL

        Returns:
            bool: 是否已采集
        """
        if not self.store:
            return False
        try:
            old = self.store.query_one_collect_url(url)
            if old:
                self.logger.debug(f"URL已采集: {url}")
                return True
            return False
        except Exception as e:
            self.logger.error(f"检查URL采集状态失败: {e}")
            return False

    def _check_content(self, content: str) -> bool:
        """检查内容是否包含关键词

        Args:
            content: 内容文本

        Returns:
            bool: 是否包含关键词
        """
        if not content:
            return False

        # 使用当前正在处理的关键词进行匹配
        if self._keyword in content:
            self.logger.info(f"内容包含关键词: {self._keyword}")
            return True

        return False

    def _save_db(
        self,
        url: str,
        content: str,
        data_type: int = 0,
        attach_str: str = None,
        is_invalid: bool = False,
    ) -> bool:
        """保存数据到数据库

        Args:
            url: 数据URL
            content: 数据内容
            data_type: 数据类型(0:招标,1:中标)
            attach_str: 附件路径
            is_invalid: 是否无效数据

        Returns:
            bool: 是否保存成功
        """
        if not self.store:
            self.logger.info(f"未设置存储器: {url}")
            return False

        try:
            status = 2 if is_invalid else 0
            data = CollectData(
                url=url,
                keyword=self.keyword,
                content=content,
                data_type=data_type,
                attach_path=attach_str,
                status=status,
                create_time=datetime.now(),
            )
            self.store.insert_collect_data(
                data, utils.get_config_bool(self.batch_save_key)
            )
            return True
        except Exception as e:
            self.logger.error(f"保存数据失败: {e}")
            return False

    def cleanup(self):
        """清理资源"""
        try:
            if self.driver:
                self.driver.quit()
        except Exception as e:
            self.logger.error(f"清理资源失败: {e}")

    @abstractmethod
    def login(self, username: str, password: str) -> None:
        """登录网站

        Args:
            username: 用户名
            password: 密码
        """
        pass

    @abstractmethod
    def _collect(self, keyword: str) -> None:
        """采集数据

        Args:
            keyword: 搜索关键词
        """
        pass
